'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _child_process = require('child_process');

var _child_process2 = _interopRequireDefault(_child_process);

var _webpack = require('./webpack.config');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// Should match the text string used in `server.listen(...)`
var RUNNING_REGEXP = /The server is running at http:\/\/(.*?)\//; /**
                                                                   * Created by Zhengfeng.Yao on 16/9/22.
                                                                   */

var server = void 0;

function nodeServer(options) {
  var webpackConfig = (0, _webpack.getWebpackConfig)(options);

  var _webpackConfig$find = webpackConfig.find(function (x) {
    return x.target === 'node';
  }),
      output = _webpackConfig$find.output;

  var serverPath = _path2.default.join(output.path, output.filename);

  return function (cb) {
    function onStdOut(data) {
      var time = new Date().toTimeString();
      var match = data.toString('utf8').match(RUNNING_REGEXP);

      process.stdout.write(time.replace(/.*(\d{2}:\d{2}:\d{2}).*/, '[$1] '));
      process.stdout.write(data);

      if (match) {
        server.stdout.removeListener('data', onStdOut);
        server.stdout.on('data', function (x) {
          return process.stdout.write(x);
        });
        if (cb) {
          cb(null, match[1]);
        }
      }
    }

    if (server) {
      server.kill('SIGTERM');
    }

    server = _child_process2.default.spawn('node', [serverPath], {
      env: Object.assign({ NODE_ENV: 'development' }, process.env),
      silent: false
    });

    server.stdout.on('data', onStdOut);
    server.stderr.on('data', function (x) {
      return process.stderr.write(x);
    });
  };
}

process.on('exit', function () {
  if (server) {
    server.kill('SIGTERM');
  }
});

exports.default = nodeServer;
module.exports = exports['default'];