'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; }; /**
                                                                                                                                                                                                                                                                               * Created by Zhengfeng.Yao on 16/9/22.
                                                                                                                                                                                                                                                                               */


exports.getBaseConfig = getBaseConfig;
exports.getBabelWebpackConfig = getBabelWebpackConfig;
exports.getWebpackConfig = getWebpackConfig;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _babel = require('./babel.config');

var _babel2 = _interopRequireDefault(_babel);

var _webpackMerge = require('webpack-merge');

var _webpackMerge2 = _interopRequireDefault(_webpackMerge);

var _extractTextWebpackPlugin = require('extract-text-webpack-plugin');

var _extractTextWebpackPlugin2 = _interopRequireDefault(_extractTextWebpackPlugin);

var _progressBarWebpackPlugin = require('progress-bar-webpack-plugin');

var _progressBarWebpackPlugin2 = _interopRequireDefault(_progressBarWebpackPlugin);

var _utils = require('./utils');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _toConsumableArray(arr) { if (Array.isArray(arr)) { for (var i = 0, arr2 = Array(arr.length); i < arr.length; i++) { arr2[i] = arr[i]; } return arr2; } else { return Array.from(arr); } }

var cwd = process.cwd();

function getBaseConfig(dev, verbose, autoprefixer) {
  autoprefixer = autoprefixer || ['last 2 versions', '> 1%', 'Explorer >= 9'];
  return {
    cache: !!dev,
    debug: !!dev,

    stats: {
      colors: true,
      reasons: !!dev,
      hash: !!verbose,
      version: !!verbose,
      timings: true,
      chunks: !!verbose,
      chunkModules: !!verbose,
      cached: !!verbose,
      cachedAssets: !!verbose
    },

    node: {
      console: false,
      global: false,
      process: false,
      Buffer: false,
      __filename: false,
      __dirname: false
    },

    plugins: [new _progressBarWebpackPlugin2.default(), new _webpack2.default.optimize.OccurenceOrderPlugin()],

    resolve: {
      extensions: ['', '.webpack.js', '.web.js', '.json', '.json5'],
      modulesDirectories: ['node_modules', _path2.default.join(cwd, 'node_modules'), _path2.default.join(__dirname, '../node_modules')]
    },

    resolveLoader: {
      modulesDirectories: ['node_modules', _path2.default.join(cwd, 'node_modules'), _path2.default.join(__dirname, '../node_modules')]
    },

    module: {
      loaders: [{
        test: /\.json$/,
        loader: 'json-loader'
      }, {
        test: /\.json5$/,
        loader: 'json5-loader'
      }, {
        test: /\.txt$/,
        loader: 'raw-loader'
      }, {
        test: /\.woff(\?v=\d+\.\d+\.\d+)?$/,
        loader: 'url?limit=10000&minetype=application/font-woff'
      }, {
        test: /\.woff2(\?v=\d+\.\d+\.\d+)?$/,
        loader: 'url?limit=10000&minetype=application/font-woff'
      }, {
        test: /\.ttf(\?v=\d+\.\d+\.\d+)?$/,
        loader: 'url?limit=10000&minetype=application/octet-stream'
      }, {
        test: /\.eot(\?v=\d+\.\d+\.\d+)?$/,
        loader: 'file'
      }, {
        test: /\.svg(\?v=\d+\.\d+\.\d+)?$/,
        loader: 'url?limit=10000&minetype=image/svg+xml'
      }, {
        test: /\.(png|jpg|jpeg|gif)(\?v=\d+\.\d+\.\d+)?$/i,
        loader: 'url?limit=10000'
      }]
    },

    postcss: function postcss() {
      return [require('postcss-nested')(), require('pixrem')(), require('autoprefixer')(autoprefixer), require('postcss-flexibility')(), require('postcss-discard-duplicates')()];
    }
  };
}

function style(dev, web, loader) {
  var query = (!!dev ? 'sourceMap&' : 'minimize&') + 'modules&localIdentName=[local]';
  if (web) {
    return _extractTextWebpackPlugin2.default.extract('style', 'css?' + query + '!' + loader);
  } else {
    return 'css/locals?' + query + '!' + loader;
  }
}

function getCommonWebpackConfig(dev, web, options, verbose) {
  var target = web ? 'web' : 'node';
  var pkg = (0, _utils.loadPackage)();
  var theme = {};
  if (pkg.theme && typeof pkg.theme === 'string') {
    var cfgPath = pkg.theme;
    // relative path
    if (cfgPath.charAt(0) === '.') {
      cfgPath = resolve(args.cwd, cfgPath);
    }
    var getThemeConfig = require(cfgPath);
    theme = getThemeConfig();
  } else if (pkg.theme && _typeof(pkg.theme) === 'object') {
    theme = pkg.theme;
  }
  var query = '{"sourceMap": true, "modifyVars": ' + JSON.stringify(theme) + '}';
  return {
    devtool: web ? !!dev ? 'cheap-module-eval-source-map' : false : 'source-map',
    target: target,
    module: {
      loaders: [{
        test: /\.less$/,
        loader: style(!!dev, web, 'postcss!less?' + query)
      }, {
        test: /\.css$/,
        loader: style(!!dev, web, 'postcss')
      }].filter(_utils.isValid)
    },
    plugins: [!dev && new _webpack2.default.NoErrorsPlugin(), new _webpack2.default.DefinePlugin({
      'process.env.NODE_ENV': !!dev ? '"development"' : '"production"',
      __DEV__: !!dev,
      'process.env.BROWSER': web,
      __BROWSER__: web
    })].concat(_toConsumableArray(web ? [new _extractTextWebpackPlugin2.default(options.fileName ? options.fileName + '.css' : !!dev ? '[name].css?[hash]' : '[name].[hash].css')].concat(_toConsumableArray(!dev ? [new _webpack2.default.optimize.DedupePlugin(), new _webpack2.default.optimize.UglifyJsPlugin({
      compress: {
        warnings: !!verbose
      }
    }), new _webpack2.default.optimize.AggressiveMergingPlugin()] : [])) : [new _webpack2.default.BannerPlugin('require("source-map-support").install();', { raw: true, entryOnly: false })])).filter(_utils.isValid)
  };
}

function getBabelWebpackConfig() {
  return {
    resolve: {
      extensions: ['', '.js', '.jsx']
    },
    babel: (0, _babel2.default)(),
    module: {
      loaders: [{
        test: /\.js$/,
        exclude: [/\.es5\.js$/, /node_modules/],
        loader: 'babel-loader'
      }, {
        test: /\.jsx$/,
        loader: 'babel-loader'
      }, {
        test: /\.est$/,
        loader: 'babel-loader!template-string-loader'
      }]
    }
  };
}

function genConfig(dev, web, options, verbose) {
  var commonConfig = getCommonWebpackConfig(dev, web, options, verbose);
  var extendConfig = getBabelWebpackConfig();
  return (0, _webpackMerge2.default)(commonConfig, extendConfig, options);
}

function getWebpackConfig(options) {
  var dev = options.dev,
      verbose = options.verbose;

  var aegisConfig = (0, _utils.loadAegisConfig)(dev ? '.dev' : '');
  var baseConfig = getBaseConfig(dev, verbose, aegisConfig.autoprefixer);
  var web = aegisConfig.web,
      node = aegisConfig.node;

  if (!web && !node) {
    console.log('Do not find client or node config.');
    return [];
  }
  var webpackConfigs = [];
  if (web) {
    webpackConfigs.push((0, _webpackMerge2.default)(baseConfig, genConfig(dev, true, web, verbose)));
  }
  if (node) {
    webpackConfigs.push((0, _webpackMerge2.default)(baseConfig, genConfig(dev, false, node, verbose)));
  }
  return webpackConfigs;
}